# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
# modifié par Cadoles 2012
#
# connectes.py
#
# Librairie de gestion des connectés
#
###########################################################################

import MySQLdb
import sys, os, tdb, time #, struct
sys.path.append('/usr/share/eole/controlevnc')

from scribe.login import Ldap, log_connexion_db
from scribe.ldapconf import USER_FILTER
from pyeole.diagnose import test_tcp
from config import mysql_password, mysql_host, master_ip

tdb_file = '/var/run/samba/sessionid.tdb'


class Session:
    def __init__(self):
        """lit le contenu de sessionid.tdb
        """
        t = tdb.Tdb(tdb_file, os.O_RDONLY)
        self.sessions = []
        ips = set()
        for key in t.iterkeys():
            #key must endswith \x00
            data = t.get(key)
            if data is None:
                continue

            #(tid, ) = struct.unpack("<L", data[0:4])
            tret = []
            data = data[4:]
            try:
                for i in range(0, 4):
                    (name, data) = data.strip('\0').split("\0", 1)
                    if i != 0:
                        tret.append(name)
                #[USER, IP, NETBIOS]
                ips.add(tret[1])
            except:
                pass
        t.close()
        db = MySQLdb.connect(host=mysql_host, user='controlevnc',
                passwd=mysql_password, db='controlevnc')
        cursor = db.cursor()
        for ip in ips:
            cursor.execute("SELECT user,netbios FROM log WHERE ip='%s'" % ip)
            ret = cursor.fetchone()
            for i in range(100):
                time.sleep(0.2)
                cursor.execute("SELECT user,netbios FROM log WHERE ip='%s'" % ip)
                ret = cursor.fetchone()
                if ret: break
            if not ret: continue
            user, netbios = ret
            self.sessions.append([user, ip, netbios])
        db.close()

    def get_users_by_ip(self, ip):
        """renvoi users par ip
        """
        ret = []
        for uid, _ip, netbios in self.sessions:
            if _ip == ip:
                ret.append(uid)
        return ret

    def get_sessions_by_ip(self, ip):
        """renvoi les sessions d'une IP
        """
        ret = []
        for uid, _ip, netbios in self.sessions:
            if _ip == ip:
                ret.append((uid, netbios, _ip))
        return ret

    def has_session(self, ip):
        for user, _ip, netbios in self.sessions:
            if _ip == ip:
                return ip

    def get_ip(self, user):
        """retourne l'ip de la (des) station(s) ou est connecte <user>
        """
        res = []
        connexions = Connexions()
        for _user, ip, netbios in self.sessions:
            if _user == user:
                #FIXME : optimal ? fait un query à chaque fois
                #et si n'existe pas ?
                if connexions.get_os_from_ip(ip) != 'Win95':
                    res.append(ip)
        return res

    def get_netbios(self, ip):
        for user, _ip, netbios in self.sessions:
            if _ip == ip:
                return netbios
        return ''

class Connexions:
    def __init__(self):
        self.db = MySQLdb.connect(host=mysql_host, user='controlevnc',
                passwd=mysql_password, db='controlevnc')
        self.cursor = self.db.cursor()
        self.sessions = None

    def __del__(self):
        self.db.commit()
        self.db.close()

    def service_start(self, ip):
        """met True à up dans la base de donnée pour l'IP correspondante
        """
        self.cursor.execute("SELECT up FROM log WHERE ip='%s'" % ip)
        row = self.cursor.fetchone()
        if row:
            if row[0] != True:
                #si l'ip est déjà dans la base n'est pas up=True
                self.cursor.execute("UPDATE log SET up=True WHERE ip='%s'" % ip)
        else:
            #si l'IP n'existe pas dans la base, creer la machine
            self.cursor.execute("INSERT INTO log (ip, up) VALUES ('%s', True)" \
                    % ip)
        return True

    def service_stop(self, ip):
        """met False à up dans la base de donnée pour l'IP correspondante
        """
        self.cursor.execute("SELECT up FROM log WHERE ip='%s'" % ip)
        row = self.cursor.fetchone()
        if row:
            if row[0] != False:
                #si l'ip est déjà dans la base n'est pas up=False
                self.cursor.execute("UPDATE log SET up=False WHERE ip='%s'"%ip)
        return True

    def get_station_up(self, verify):
        """
        retourne la liste des machines dans la base de donnée des machines
        avec session ouverte (information du Client Scribe).
        Si verify (True|False):
        - comparaison avec la liste des sessions samba ;
        - test si la présence du port 8788 (Client Scribe), dans ce cas le nom
        de l'utilisation n'est pas mentionné).
        """
        #FIXME : devrait ajouter les machines de la session nom présente dans
        #bdd
        self.cursor.execute("SELECT ip, netbios, user FROM log WHERE up=True")
        ret = []
        session_ips = [ip for user, ip, netbios in self.get_sessions().sessions]
        for row in self.cursor.fetchall():
            #si le poste a une session samba ou est accessible sur le port 8788
            if not verify or row[0] in session_ips:
                ret.append(', '.join(row))
            elif test_tcp(row[0], 8788):
                ret.append("%s, %s," % (row[0], row[1]))
        return ';'.join(ret)

    def get_connected_by_group(self, group):
        """voir les connecter pour un groupe"""
        #FIXME manque UP
        sql = "SELECT ip, user, os, prim_group FROM log WHERE ip IN"
        sql += "(SELECT ip FROM `group` WHERE groupname='%s');" % group
        self.cursor.execute(sql)
        members = []
        ip_infos = {}
        for row in self.cursor.fetchall():
            ip, user, os, prim_group = row
            members.append(user)
            ip_infos[ip] = (os, prim_group)

        sessions = self.get_sessions()
        #for username, prigrp, nom_mach, os, ip in sessions.sessions:
        liste = []
        #FIXME : supprimer les comptes en trop dans la bdd !
        for username, ip, nom_mach in sessions.sessions:
            try:
                # uid, grp, machine, os(détecté), ip
                if username not in members:
                    continue
                os, prigrp = ip_infos[ip]

                liste.append((username, prigrp, nom_mach, os, ip))
            except:
                continue
        liste.sort()
        return liste

    def get_connected_student(self, group):
        """renvoie la liste des connectes pour n'avoir que les élèves du
        groupe [USERNAME]
        """
        # retourne que les eleves
        prigrp = "eleves"
        # récupération des connectés
        #croise parce que cherche prigrp eleves + groupe + connecté
        """retourne les utilisateurs présent dans 2 groupes"""
        #FIXME manque UP
        sql = "SELECT ip, os, user FROM log WHERE ip IN "
        sql += "(SELECT ip FROM `group` WHERE EXISTS"
        sql += "(SELECT ip FROM `group` WHERE groupname='%s')" % prigrp
        sql += "AND groupname='%s');" % group
        self.cursor.execute(sql)
        membres = []
        ip_infos = {}
        for row in self.cursor.fetchall():
            ip, os, user = row
            membres.append(user)
            ip_infos[ip] = os
        # le groupe est vide
        if not membres:
            return []
        sessions = self.get_sessions()
        liste = []
        for username, ip, nom_mach in sessions.sessions:
            try:
                # uid, grp, machine, os(détecté), ip
                if username not in membres:
                    continue
                os = ip_infos[ip]

                liste.append((username, prigrp, nom_mach, os, ip))
            except:
                continue
        liste.sort()
        return liste

    def get_os_from_ip(self, ip):
        sql = "SELECT os FROM log WHERE ip='%s'" % ip
        self.cursor.execute(sql)
        row = self.cursor.fetchone()
        if row:
            return row[0]

    def get_ip_from_netbios(self, netbios):
        sql = "SELECT ip FROM log WHERE netbios='%s'" % netbios
        self.cursor.execute(sql)
        row = self.cursor.fetchone()
        if row:
            return row[0]

    def get_netbios_from_ip(self, ip):
        sql = "SELECT netbios FROM log WHERE ip='%s'" % ip
        self.cursor.execute(sql)
        row = self.cursor.fetchone()
        if row:
            return row[0]

    def get_netbios(self):
        sql = "SELECT netbios FROM log"
        self.cursor.execute(sql)
        ret = []
        row = self.cursor.fetchall()
        if row:
            for r in row:
                ret.append(r[0])
        return ret

    def get_sid(self, user):
        #FIXME manque UP
        sql = "SELECT sid FROM log WHERE user='%s'" % user
        self.cursor.execute(sql)
        row = self.cursor.fetchone()
        if row:
            return row[0]
        #si pas dans la base de donnée
        return Ldap().ldap_search("(&%s(uid=%s))" % (USER_FILTER, user),
                ['sambaSID'], True)['sambaSID'][0]

    def get_sessions(self):
        if not self.sessions:
            self.sessions = Session()
        return self.sessions

    def get_infos(self, ip, os_type):
        sessions = self.get_sessions()
        users = sessions.get_users_by_ip(ip)

        #récupération des informations sur l'utilisateur dans la base de données
        #FIXME manque UP
        sql = "SELECT user, sid, display_name, os, netbios FROM log "
        sql += "WHERE ip='%s'" % ip
        self.cursor.execute(sql)
        row = self.cursor.fetchone()
        if row:
            # Si personne dans la session
            # ou un et un seul utilisateur identique à la base de donnée
            # ou l'utilisateur de la base est dans la liste des sessions
            # (il arrive que net status sessions renvoi plusieurs utilisateurs
            # retourne les valeurs de la base
            user, sid, display_name, _os, netbios = row
            if users == [] or (len(users) == 1 and users[0] == user) or \
                    (len(users) > 1 and user in users):
                #si pas vide
                if _os:
                    groups = self._get_sql_groups(ip)
                    return user, sid, display_name, groups, _os, netbios
        # si pas d'utilisateur dans la base de donnée
        # et dans net sessions status
        if users == []:
            raise Exception("Personne ne s'est connecte sur %s"%ip)
        # utilisateur de la base et connecté différent
        # ou pas d'utilisateur dans la base de donnée
        # fait confiance à net status sessions
        # on récupère les informations dans LDAP
        user = users[-1]

        ldap_conn = Ldap()
        user_attrib = ldap_conn.get_user_attributs(user)
        sid = user_attrib['sambaSID'][0].strip()
        display_name = user_attrib['displayName'][0].strip()
        gidnumber = user_attrib['gidNumber'][0].strip()
        groups = []
        for dn, grp in ldap_conn.get_groups(user):
            name = grp['cn'][0]
            if grp['gidNumber'][0] == gidnumber:
                prim_group = name
            groups.append(name)
        del(ldap_conn)
        netbios = sessions.get_netbios(ip)

        # met à jour la base de donnée
        log_connexion_db(user, sid, display_name, prim_group, groups, netbios,
                os_type, ip)

        return user, sid, display_name, groups, os_type, netbios

    def _get_sql_groups(self, ip):
        """retourne les groupes de l'utilisateur connecté à une IP sans
        vérifier l'utilisateur"""
        #FIXME manque UP
        sql = "SELECT groupname FROM `group` WHERE ip='%s'" % ip
        self.cursor.execute(sql)
        groups = [group[0] for group in self.cursor.fetchall()]
        return groups

    def get_groups(self, ip):
        """retourne les groupes de l'utilisateur connecter sur IP
        """
        sessions = self.get_sessions()
        users = sessions.get_users_by_ip(ip)

        #récupération des informations sur l'utilisateur dans la base de données
        #FIXME manque UP
        sql = "SELECT user FROM log WHERE ip='%s'" % ip
        self.cursor.execute(sql)
        user = self.cursor.fetchone()
        if user:
            # Si personne dans la session
            # ou un et un seul utilisateur identique à la base de donnée
            # ou l'utilisateur de la base est dans la liste des sessions
            # (il arrive que net status sessions renvoi plusieurs utilisateurs
            # retourne les valeurs de la base
            if users == [] or (len(users) == 1 and users[0] == user) or \
                    (len(users) > 1 and user in users):
                #si pas vide
                return self._get_sql_groups(ip)

        # si pas d'utilisateur dans la base de donnée
        # et dans net sessions status
        if users == []:
            raise Exception("Personne ne s'est connecte sur %s"%ip)
        # utilisateur de la base et connecté différent
        # ou pas d'utilisateur dans la base de donnée
        # fait confiance à net status sessions
        # on récupère les informations dans LDAP
        user = users[-1]

        ldap_conn = Ldap()
        groups = []
        for dn, grp in ldap_conn.get_groups(user):
            groups.append(grp['cn'][0])
        del(ldap_conn)

        return groups

    def get_user(self, ip):
        """retourne l'utilisateur connecter sur IP
        """
        sessions = self.get_sessions()
        users = sessions.get_users_by_ip(ip)

        #récupération des informations sur l'utilisateur dans la base de données
        #FIXME manque UP
        sql = "SELECT user FROM log WHERE ip='%s'" % ip
        self.cursor.execute(sql)
        row = self.cursor.fetchone()
        if row:
            user = row
            # Si personne dans la session
            # ou un et un seul utilisateur identique à la base de donnée
            # ou l'utilisateur de la base est dans la liste des sessions
            # (il arrive que net status sessions renvoi plusieurs utilisateurs
            # retourne les valeurs de la base
            if users == [] or (len(users) == 1 and users[0] == user) or \
                    (len(users) > 1 and user in users):
                #si pas vide
                return user

        # si pas d'utilisateur dans la base de donnée
        # et dans net sessions status
        if users == []:
            raise Exception("Personne ne s'est connecte sur %s"%ip)
        # utilisateur de la base et connecté différent
        # ou pas d'utilisateur dans la base de donnée
        # fait confiance à net status sessions
        # on récupère les informations dans LDAP
        return users[-1]

    def isprof(self, ip):
        """renvoie True si l'utilisateur connecté sur <ip> est membre de
        "professeurs"
        """
        if ip == master_ip:
            return True
        return 'professeurs' in self.get_groups(ip)

    def isadmin(self, ip):
        """renvoie True si l'utilisateur connecté sur <ip> est membre de
        "DomainAdmins"
        """
        if ip == master_ip:
            return True
        return 'DomainAdmins' in self.get_groups(ip)

    def has_session(self, ip):
        sessions = self.get_sessions()
        return sessions.has_session(ip)

